######Uppsala R workshop - Day 2######


###code block B1

library(MCMCglmm)
data(BTdata)
data(BTped)

#lets look at some chosen family

parents <- function(x) {
		any(x=="R187920"|x=="R187921")
		}

where<-apply(BTped,1,parents)
fam <- BTped[which(where),]
fam

#to see the monster - just calculate the relationships
install.packages("kinship"); library(kinship)

#multiply by 2 since relatedness is twice P
#that an allele is ibd in 2 individuals
A <- 2*kinship(fam[,1],fam[,2],fam[,3])
A


###code block B2

prior.b1 <- list(R=list(V=1,nu=0.002),G=list(G1=list(V=1,nu=0.002),
		G2=list(V=1,nu=0.002),G3=list(V=1,nu=0.002)))

m.b1 <- MCMCglmm(tarsus~sex,random=~animal+dam+fosternest,
		data=BTdata,pedigree=BTped,verbose=F,prior=prior.b1)
plot(m.b1$VCV)#needs attention - poor mixing
# may take up to 5 min!
m.b1 <- MCMCglmm(tarsus~sex,random=~animal+dam+fosternest,
		data=BTdata,pedigree=BTped,verbose=F,prior=prior.b1,
		nitt=100000,burnin=30000,thin=50)
plot(m.b1$VCV)

#perhaps simplification is a better choice
#since we have only full-sibs in our data
#genetic effects may be confounded with
#the dam effect and hence this poor mixin

m.b2 <- MCMCglmm(tarsus~sex,random=~animal+fosternest,
		data=BTdata,pedigree=BTped,verbose=F,prior=prior.b1,
		nitt=30000,burnin=10000,thin=30)
plot(m.b2$VCV) #much better!
plot(m.b2$Sol)

diag(autocorr(m.b2$VCV)[2,,])
# correlations are a bit high so we might run model for longer
# i'll leavy explaration to you to save our class time
# i'd recommend running this model for at least 100000 iterations

HPDinterval(m.b2$VCV[,1]) #CI for VA
h2 <- m.b2$VCV[,"animal"]/rowSums(m.b2$VCV)
posterior.mode(h2)
HPDinterval(h2)

# try at home, may take up to 10 min to calculate
m.b2best <- MCMCglmm(tarsus~sex,random=~animal+fosternest,
		data=BTdata,pedigree=BTped,verbose=F,prior=prior.b1,
		nitt=100000,burnin=30000,thin=100)
diag(autocorr(m.b2best$VCV)[2,,]) #much better



###code block B3

# for broad sense heritability
m.b2a <- MCMCglmm(tarsus~sex,random=~dam+fosternest,
		data=BTdata,verbose=F,prior=prior.b1)
plot(m.b2a)
H2 <- 2*(m.b2a$VCV[,"dam"]/rowSums(m.b2a$VCV))
posterior.mode(H2)
HPDinterval(H2)


###code block B4

gryph<-read.table("Gryphon.txt",head=T,sep="\t")
gryped<-read.table("gryphonped.txt",head=T,sep="\t")


names(gryph)[1]<-"animal"
gryph$animal <- as.factor(gryph$animal)
gryph$MOTHER <- as.factor(gryph$MOTHER)
gryph$BYEAR <- as.factor(gryph$BYEAR)
gryph$SEX <- as.factor(gryph$SEX)
gryph$BWT <- as.numeric(gryph$BWT)
gryph$TARSUS <- as.numeric(gryph$TARSUS)

#factorise all ids in pedigree
for (i in 1:3) {gryped[,i]<-as.factor(gryped[,i])}

m.b31 <- MCMCglmm(TARSUS~1, random=~animal, data=gryph,
		pedigree=gryped, verbose=F)
plot(m.b31$VCV)

VTAR <- var(gryph$TARSUS,na.rm=T)
prior.b31<- list(R=list(V=VTAR/2,nu=1),G=list(G1=list(V=VTAR/2,nu=1)))
m.b31 <- MCMCglmm(TARSUS~1, random=~animal, data=gryph,
		pedigree=gryped, verbose=F, nitt=50000, burnin=10000,
		thin=50, prior=prior.b31)
plot(m.b31$VCV) #much better although even longer=better

#heritability
h21 <- m.b31$VCV[,"animal"]/rowSums(m.b31$VCV)
posterior.mode(h21)
HPDinterval(h21)


m.b32 <- MCMCglmm(TARSUS~SEX, random=~animal, data=gryph,
		pedigree=gryped, verbose=F, nitt=50000, burnin=10000,
		thin=50, prior=prior.b31)
plot(m.b32$VCV) #much better although even longer=better

#heritability
h22 <- m.b32$VCV[,"animal"]/rowSums(m.b32$VCV)
posterior.mode(h22)
HPDinterval(h22)
#sex took some VR and thus increased h2

prior.b33<- list(R=list(V=VTAR/3,nu=1),G=list(G1=list(V=VTAR/3,nu=1),
			G2=list(V=VTAR/3,nu=1)))
m.b33 <- MCMCglmm(TARSUS~SEX, random=~animal+BYEAR, data=gryph,
		pedigree=gryped, verbose=F, nitt=50000, burnin=10000,
		thin=50, prior=prior.b33)
plot(m.b33$VCV) #much better although even longer=better

#heritability
h23 <- m.b33$VCV[,"animal"]/rowSums(m.b33$VCV)
posterior.mode(h23)
HPDinterval(h23)
posterior.mode(m.b32$VCV[,"units"])
posterior.mode(m.b33$VCV[,"units"]+m.b33$VCV[,"BYEAR"])


prior.b34<- list(R=list(V=VTAR/4,nu=1),G=list(G1=list(V=VTAR/4,nu=1),
			G2=list(V=VTAR/4,nu=1),G3=list(V=VTAR/4,nu=1)))
m.b34 <- MCMCglmm(TARSUS~SEX, random=~animal+BYEAR+MOTHER, data=gryph,
		pedigree=gryped, verbose=F, nitt=50000, burnin=10000,
		thin=50, prior=prior.b34)
plot(m.b34$VCV)

#heritability
h24 <- m.b34$VCV[,"animal"]/rowSums(m.b34$VCV)
posterior.mode(h24)
HPDinterval(h24)
#here heritability of TARSUS dropped dramatically as we used
#another random effect that accounts for similarity between sibs


###code block B5


gryrm<-read.table("gryphonRM.txt",head=T,sep="\t")
names(gryrm)[1]<-"animal"
gryrm$animal<-as.factor(gryrm$animal)
gryrm$BYEAR<-as.factor(gryrm$BYEAR)
gryrm$AGE<-as.factor(gryrm$AGE)
gryrm$YEAR<-as.factor(gryrm$YEAR)
gryrm$LAYDATE<-as.numeric(gryrm$LAYDATE)
gryrm$ID<-gryrm$animal

VLAY <- var(gryrm$LAYDATE, na.rm=T)
prior.b41 <- list(R=list(V=VLAY/2,nu=1),G=list(G1=list(V=VLAY/2,nu=1)))
m.b41 <- MCMCglmm(LAYDATE~1,random=~ID,verbose=F,data=gryrm,
			prior=prior.b41)
plot(m.b41$VCV)
#repeatability
rep1 <- m.b41$VCV[,"ID"]/rowSums(m.b41$VCV)
posterior.mode(rep1)
HPDinterval(rep1)

#lets account from age differences
m.b42 <- MCMCglmm(LAYDATE~AGE,random=~ID,verbose=F,data=gryrm,
			prior=prior.b41)
plot(m.b42$VCV)
#repeatability
rep2 <- m.b42$VCV[,"ID"]/rowSums(m.b42$VCV)
posterior.mode(rep2)
HPDinterval(rep2)

#naive heritability
m.b43 <- MCMCglmm(LAYDATE~AGE,random=~animal,verbose=F,data=gryrm,
			prior=prior.b41,pedigree=gryped)
plot(m.b43$VCV)
#heritability
h25 <- m.b43$VCV[,"animal"]/rowSums(m.b43$VCV)
posterior.mode(h25)
HPDinterval(h25)

#heritability + permanet env effects
prior.b41 <- list(R=list(V=VLAY/3,nu=1),G=list(G1=list(V=VLAY/3,nu=1),
			G2=list(V=VLAY/3,nu=1)))
m.b44 <- MCMCglmm(LAYDATE~AGE,random=~animal+ID,verbose=F,data=gryrm,
			prior=prior.b41,pedigree=gryped)
plot(m.b44$VCV)#be careful - running for longer required
#heritability and repeatability
h26 <- m.b44$VCV[,"animal"]/rowSums(m.b44$VCV)
rep2 <- m.b44$VCV[,"ID"]/rowSums(m.b44$VCV)
posterior.mode(h26)
HPDinterval(h26)
posterior.mode(rep2)
HPDinterval(rep2)


###code block B6

mammals <- read.nexus("mammals.nex")
mammals
summary(mammals)
mammals.plot<-plot(mammals,font=1,cex=0.55)
nodelabels()
mammals$edge
#if you want you can write the tree in newick format
write.tree(mammals,file="mammals.nck")


###code block B7

data(woodmouse)
woodmouse
base.freq(woodmouse)
write.dna(woodmouse,"woodmouse.fas",format="fasta")
rodents <- read.dna("woodmouse.fas",format="fasta")

rodents[1,] #first sequence
as.character(rodents[1,1:50])
paste(as.character(rodents[1,1:50]),collapse="")

#calculate phylogeny based on these sequences
dist.dna(rodents[1:5,])
rodents.dist<-dist.dna(rodents)
as.matrix(dist.dna(rodents[1:5,])) # looks much better

#build a tree using UPGMA
cluster<-hclust(rodents.dist)
rodents.upgma<-as.phylo(cluster)
plot(rodents.upgma,cex=0.75,font=1,no.margin=T)

#we can use neighbour joining instead
cluster.nj<-nj(rodents.dist)
rodents.nj<-as.phylo(cluster.nj)
plot(rodents.nj,cex=0.75,font=1,no.margin=T)

#additionally we can use Gascuel's improved NJ
cluster.bionj<-bionj(rodents.dist)
rodents.bionj<-as.phylo(cluster.bionj)
plot(rodents.bionj,cex=0.75,font=1,no.margin=T)

#note that we can use different evolution models- see ?dist.dna
par(mfrow=c(2,2))
plot(as.phylo(nj(dist.dna(rodents,model="JC69"))),cex=0.75,font=1,no.margin=T)
plot(as.phylo(nj(dist.dna(rodents,model="K81"))),cex=0.75,font=1,no.margin=T)
plot(as.phylo(nj(dist.dna(rodents,model="F84"))),cex=0.75,font=1,no.margin=T)
plot(as.phylo(nj(dist.dna(rodents,model="GG95"))),cex=0.75,font=1,no.margin=T)

par(mfrow=c(1,1)) #to restore default graphic parameters


###codeblock B8

#compare trees
all.equal(rodents.nj,rodents.bionj)
#and ignoring branch lengths - i.e. comparring only topologies
all.equal(rodents.nj,rodents.bionj,use.edge.length=F)

#having a lot of trees you can calculate distances between them
dist.topo(rtree(30),rtree(30))


###code block B9

rodents.boot <- boot.phylo(rodents.nj,rodents,function(x){nj(dist.dna(x))},
			B=200,block=1)
rodents.boot/2
plot(rodents.nj)
nodelabels(rodents.boot/2)


###code block B10

rodents2 <- read.tree("rodents.tre")
summary(rodents2)
mouseData <- rnorm(14,mean=10,sd=sqrt(5))
mouseCont <- pic(mouseData,rodents2)
mouseCont #remember to supress intercept using PICs

vcv.phylo(rodents2)[1:3,1:3]


###code block B11

#sample data on bird families
data(bird.families)
bird.families$tip.label[1:20]
bird.families<-makeNodeLabel(bird.families)#naming all ancestral nodes
#let's choose 4 families and extract frm the phylogeny
some.families<-c("Certhiidae","Paridae","Gruidae","Struthionidae")
bird.fam.4 <- drop.tip(bird.families, setdiff(bird.families$tip.label,
			some.families))
vcv.phylo(bird.fam.4,cor=T)
plot(bird.fam.4)
plot(bird.families,font=1,cex=0.4)

bird.fam.30 <- drop.tip(bird.families, setdiff(bird.families$tip.label,
			bird.families$tip.label[1:20]))
plot(bird.fam.30)
plot(bird.fam.30,type="cladogram")
plot(bird.fam.30,type="fan")
plot(bird.fam.30,type="unrooted")
plot(bird.fam.30,type="radial")


###code block B12

data(carnivora)
carphyl <- read.tree("carnphyl.tre")
plot(carphyl)
carphyl$tip.label
levels(carnivora$Family)
carphyl$tip.label<-c("Felidae","Viverridae","Hyaenidae",
				"Canidae","Mustelidae","Procyonidae",
				"Ursidae","Ailuridae")
plot(carphyl)


###code block B13

summary(carnivora)

m.b51 <- MCMCglmm(FW~LS+GL+as.numeric(AM),data=na.omit(carnivora),verbose=F)
plot(m.b51$Sol)
plot(m.b51$VCV)
summary(m.b51)

names(carnivora)[3]<-"animal"
prior.b52 <- list(R=list(V=1,nu=0.002),G=list(G1=list(V=1,nu=0.002)))
m.b52 <- MCMCglmm(FW~LS+GL+as.numeric(AM),data=na.omit(carnivora),
		verbose=F,random=~animal,prior=prior.b52,pedigree=carphyl)
plot(m.b52$Sol)
plot(m.b52$VCV)
summary(m.b52)

#let's try for simulated data ob bird families

bf.sim <- rTraitCont(bird.families,sigma=runif(Nedge(bird.families),0.1,0.7))
bf.sim <- data.frame(y=bf.sim,animal=names(bf.sim))
bf.sim[1:20,]
#we'll add some residuals on top
err <- rnorm(137,sd=sqrt(3))
bf.sim[,1]<-bf.sim[,1]+err
#and replication
bf.sim2 <- as.data.frame(bf.sim[sample(1:137,50),])
err2 <- rnorm(50,runif(20,1,2),sqrt(3))
bf.sim2[,1]<-bf.sim2[,1]+err2
bf.sim3 <- as.data.frame(bf.sim[sample(1:137,50,replace=T),])
err3 <- rnorm(50,runif(20,1.5,2.5),sqrt(3))
bf.sim3[,1]<-bf.sim3[,1]+err3
#and combine the three
bf.sim<-rbind(bf.sim,bf.sim2,bf.sim3)
summary(bf.sim)

#prior is the same

m.b53 <- MCMCglmm(y~1,random=~animal,pedigree=bird.families,
			data=bf.sim,verbose=F,prior=prior.b52)
plot(m.b53$VCV)
posterior.mode(m.b53$Sol)

#not accounting for phylogenetic depndence yields false picture
m.b53a <- MCMCglmm(y~1,data=bf.sim,verbose=F)
posterior.mode(m.b53a$Sol)

#as in ordinary animal model we can estimate so 
#called phylogenetic heritability
#the proportion of total variance explained by 
#phylogenetic effects of shared ancestry

hp2 <- m.b53$VCV[,"animal"]/rowSums(m.b53$VCV)
posterior.mode(hp2)
HPDinterval(hp2)


###code block B14

data(ChickWeight)
xyplot(weight~Time|Chick,data=ChickWeight)

prior.b61 <- list(R=list(V=1e-16,nu=-2),G=list(G1=list(V=1,nu=1)))
m.b61 <- MCMCglmm(weight~Diet+poly(Time,2,raw=T),random=~Chick,
		data=ChickWeight, verbose=F, pr=T, prior=prior.b61,
		saveX=T, saveZ=T)
pop.int <- posterior.mode(m.b61$Sol[,1])
pop.slope <- posterior.mode(m.b61$Sol[,5])
pop.quad <- posterior.mode(m.b61$Sol[,6])
chick.int <- posterior.mode(m.b61$Sol[,c(7:56)])

time <- ChickWeight$Time[1:12]
plot(pop.int+pop.slope*I(time^1)+pop.quad*I(time^2)~time,
	type="l",lwd=2,ylim=c(-50,400))
for(i in 1:50) {
	lines(pop.int+chick.int[i]+pop.slope*I(time^1)+
		pop.quad*I(time^2)~time,lty=3,col="red")
}

#we can print predictions from our model for each chick by combining
#multiplying desing matrix W=[X,Z] for effects with 
#parameter vector theta=[beta,u]
W1 <- cBind(m.b61$X,m.b61$Z)
theta <- posterior.mode(m.b61$Sol)
prediction1 <- W1 %*% theta
xyplot(weight+prediction1[,1]~Time|Chick,data=ChickWeight)


###code block B15

prior.b62 <- list(R=list(V=1e-16,nu=-2),G=list(G1=list(V=diag(2),nu=2)))
m.b62 <- MCMCglmm(weight~Diet+poly(Time,2,raw=T),random=~us(1+Time):Chick,
		data=ChickWeight, verbose=F, pr=T,prior=prior.b62,saveX=T,saveZ=T)
plot(m.b62$VCV)
diag(autocorr(m.b62$VCV)[2,,])
r.int.slope <- m.b62$VCV[,2]/sqrt(m.b62$VCV[,1]*m.b62$VCV[,4])
posterior.mode(r.int.slope)
#cor close to space boundary - should be run for longer

#could do predictions by hand or like here by using predict()
xyplot(weight+predict(m.b62,marginal=NULL)~Time|Chick,data=ChickWeight)
#looks MUCH better - may could be better 
#adding second random slope for quadratic term?
prior.b63 <- list(R=list(V=1e-16,nu=-2),G=list(G1=list(V=diag(3),nu=3)))
m.b63 <- MCMCglmm(weight~Diet+poly(Time,2,raw=T),
		random=~us(1+poly(Time,2,raw=T)):Chick,
		data=ChickWeight, verbose=F, pr=T,
		prior=prior.b63,saveX=T,saveZ=T)
xyplot(weight+predict(m.b63,marginal=NULL)~Time|Chick,data=ChickWeight)

#DICs confirm its the best model - hence chicks differ both in 
#intercepts and (quadratic)slopes
m.b61$DIC;m.b62$DIC;m.b63$DIC

#to confirm we could see if REML estimators corroborate these conclusions
library(lme4)
m.b61reml <- lmer(weight~Diet+poly(Time,2,raw=T)+(1|Chick),data=ChickWeight)
summary(m.b61reml)@AICtab[1]

m.b62reml <- lmer(weight~Diet+poly(Time,2,raw=T)+(1+Time|Chick),
			data=ChickWeight)
summary(m.b62reml)@AICtab[1]

m.b63reml <- lmer(weight~Diet+poly(Time,2,raw=T)+(1+poly(Time,2,raw=T)|Chick),
			data=ChickWeight)
summary(m.b63reml)@AICtab[1]
detach(package:lme4)


###code block B16

toyslope <- rnorm(30)#30 random slopes ~N(0,1)
#prepare space for the plots
plot(0,type="n",xlim=c(-1,1),ylim=c(-3,3),ylab="y",xlab="time")
for (i in 1:30) { #for each of 30 slopes
	abline(a=0,b=toyslope[i])
}

time<-seq(0,21,length=100)
polynomial<-leg(time,2,normalized=F)
#better than poly because generates first column of ones giving
#appropriate design matrix for fixed and random slope effects

#coeficient for fixed effects
beta1 <- c(posterior.mode(m.b61$Sol[,1]),posterior.mode(m.b61$Sol[,5]),
		posterior.mode(m.b61$Sol[,6]))
beta2 <- c(posterior.mode(m.b62$Sol[,1]),posterior.mode(m.b62$Sol[,5]),
		posterior.mode(m.b62$Sol[,6]))
beta3 <- c(posterior.mode(m.b63$Sol[,1]),posterior.mode(m.b63$Sol[,5]),
		posterior.mode(m.b63$Sol[,6]))

#covariance matrices and residuals

VCV1 <- matrix(posterior.mode(m.b61$VCV)[1],1,1)#single variance
VCV2 <- matrix(posterior.mode(m.b62$VCV)[1:(2^2)],2,2)#4 parameters
VCV3 <- matrix(posterior.mode(m.b63$VCV)[1:(3^2)],3,3)#9 parameters
units1 <- posterior.mode(m.b61$VCV)[2]
units2 <- posterior.mode(m.b62$VCV)[5]#5th parameter cause 4 for (co)variances
units3 <- posterior.mode(m.b63$VCV)[10]#10th cause 9 pars for (co)variances

plot(weight~Time,data=ChickWeight,cex.lab=1.5)
mu1 <- polynomial %*% beta1 #population line across time
sd1 <- sqrt(units1+diag(polynomial[,1,drop=F]%*%
		VCV1%*%t(polynomial[,1,drop=F])))
#%*% multiplies matrices; drop lets matrix be a matrix after extracting one
#dimension, otherwise it would be a vector and would cause problems when
#trying to multiply to get ZVZ'; by using first column of polynomial we
#create 'new' Z matrix appropriate for the time sequence we have, of length
#100 rather than 12
lines(mu1~time,lwd=2)
lines(I(mu1+1.96*sd1)~time,lty=2,lwd=1,col="red")
lines(I(mu1-1.96*sd1)~time,lty=2,lwd=1,col="red")

plot(weight~Time,data=ChickWeight,cex.lab=1.5)
mu2 <- polynomial %*% beta2 #population line across time
sd2 <- sqrt(units2+diag(polynomial[,1:2,drop=F]%*%VCV2%*%
		t(polynomial[,1:2,drop=F])))
lines(mu2~time,lwd=2)
lines(I(mu2+1.96*sd2)~time,lty=2,lwd=1,col="red")
lines(I(mu2-1.96*sd2)~time,lty=2,lwd=1,col="red")
#very good fit of variance change to data

plot(weight~Time,data=ChickWeight,cex.lab=1.5,ylim=c(-150,600))
mu3 <- polynomial %*% beta3 #population line across time
sd3 <- sqrt(units2+diag(polynomial[,1:3,drop=F]%*%VCV3%*%
		t(polynomial[,1:3,drop=F])))
lines(mu3~time,lwd=2)
lines(I(mu3+1.96*sd3)~time,lty=2,lwd=1,col="red")
lines(I(mu3-1.96*sd3)~time,lty=2,lwd=1,col="red")
#very poor  fit of variance change to data, 2nd model seems the best!


###code block 17

id <- gl(200,4) #200 people, 4 records for each
id[1:50]
av.wealth<-rlnorm(200,0,1)
ac.wealth<-av.wealth[id]+rlnorm(800,0,1)
av.ratio<-rbeta(200,10,10)#beta is good distribution for ratios
ac.ratio<-rbeta(800,2*(av.ratio[id]),2*(av.ratio[id]))
y.car<-(ac.wealth*ac.ratio)^0.25
y.hol<-(ac.wealth*(1-ac.ratio))^0.25
spend<-data.frame(y.hol=y.hol,y.car=y.car,id=id)

#simple regression - are these two related?
summary(lm(y.car~y.hol,data=spend))

#accounting for random variation doesn't change the point
summary(m.b7a <- MCMCglmm(y.car~y.hol,random=~id,data=spend,verbose=F))

#is this model valid? univariate model assumes that there is causal link
#between spending money on holiday and spending it on car
#however we've model the data so that not only spending on car affects
#spending on holiday - or vice versa - but also there's some year-to-year
#variation in the income, affecting the spendings
#adopting regression ignores variation in x and hence in
#(unmeasured) income; we'll model these variabilities fitting
#both spendings as two separate traits

prior.b7<-list(R=list(V=diag(2),nu=2),G=list(G1=list(V=diag(2),nu=2)))
m.b7 <- MCMCglmm(cbind(y.hol,y.car)~trait-1, random=~us(trait):id,
		rcov=~us(trait):units,data=spend,family=c("gaussian","gaussian"),
		verbose=F,prior=prior.b7)
cor(matrix(colMeans(m.b7$VCV),2,2))
#let's look at regression coefficients describing different parts of variability
#in this data: a = Cov/Var

id.r <- m.b7$VCV[,2]/m.b7$VCV[,1]
res.r <- m.b7$VCV[,6]/m.b7$VCV[,5]
plot(mcmc.list(m.b7a$Sol[,"y.hol"],id.r,res.r),density=F,col=c("red","green","blue"))
#trends are opposite depending on the part of variation we look on
#note that if the data were generated so that relationship between spending on holidays
#and car was purely causal
spend$y.hol2<-rnorm(200,0,sqrt(2))[spend$id]+rnorm(800,0,sqrt(1))
spend$y.car2<-spend$y.hol2*-0.3+rnorm(200,0,sqrt(1))[spend$id]+rnorm(800,0,sqrt(2))

m.b7ab <- MCMCglmm(y.car2~y.hol2,random=~id,data=spend,verbose=F)
m.b7b <- MCMCglmm(cbind(y.hol2,y.car2)~trait-1, random=~us(trait):id,
		rcov=~us(trait):units,data=spend,family=c("gaussian","gaussian"),
		verbose=F,prior=prior.b7)
id.r2 <- m.b7b$VCV[,2]/m.b7b$VCV[,1]
res.r2 <- m.b7b$VCV[,6]/m.b7b$VCV[,5]
plot(mcmc.list(m.b7ab$Sol[,"y.hol2"],id.r2,res.r2),density=F,col=c("red","green","blue"))
#slopes are virtually the same up to Monte Carlo error


###code block B18

data(BTdata)
data(BTped)
#2x2 prior since we model 2 traits - tarsus and back color
prior.b81 <- list(R=list(V=diag(2),nu=1.002),G=list(G1=list(V=diag(2),nu=1.002),
			G2=list(V=1,nu=0.002)))
m.b81 <- MCMCglmm(cbind(tarsus,back)~trait+sex:trait-1,
		random=~us(trait):animal+fosternest,
		rcov=~us(trait):units,verbose=F,data=BTdata,
		pedigree=BTped,nitt=70000,burnin=15000,thin=70,
		family=c("gaussian","gaussian"),prior=prior.b81)
plot(m.b81$VCV)#traces show autocorrelation and probably nitt around 200000 would be required
#we'll go on with these results as running longer model would require >15 min!
#both back colour and tarsus seem to be genetically controlled
#but what about genetic correlation?
#rG=Cov/sqrt(Var1*Var2)
rG<-m.b81$VCV[,2]/sqrt(m.b81$VCV[,1]*m.b81$VCV[,4])
posterior.mode(rG)
HPDinterval(rG)#rG is not zero truncated so can be tested using CIs


###code block B19

prior.b82 <- list(R=list(V=diag(3),nu=2.002),G=list(G1=list(V=diag(3),nu=2.002),
			G2=list(V=1,nu=0.002)))
m.b82 <- MCMCglmm(tarsus~sex,
		random=~us(sex):animal+fosternest,
		rcov=~idh(sex):units,verbose=F,data=BTdata,
		nitt=50000,burnin=12000,thin=50,prior=prior.b82,pedigree=BTped)
posterior.mode(m.b82$VCV)
rMF<-m.b82$VCV[,2]/sqrt(m.b82$VCV[,1]*m.b82$VCV[,5])
HPDinterval(rMF)
posterior.mode(rMF)
#rG is not zero truncated so can be tested using CIs

