######Uppsala R workshop - Day 3######


###code block C1

library(MCMCglmm)
clines <- read.csv("mamm_clines.csv",head=T)
clines<-clines[,-4]
clines$FisherZ<-0.5*log((1+clines$corr)/(1-clines$corr))
#effect size
clines$mev<-1/(clines$N-3)#measurement error
clines$weight<-1/clines$mev
#weights if analysing in ordinary lm/lmer/glm/lme

prior.c1 <- list(R=list(V=1,nu=0.002))

m.c11 <- MCMCglmm(FisherZ~1, verbose=F, prior=prior.c1,
		data=clines)
summary(m.c11)

#we add sampling error of statistics mev
m.c12 <- MCMCglmm(FisherZ~1, verbose=F, prior=prior.c1,
			data=clines,
			mev=clines$mev)
summary(m.c12)
plot(m.c12)

plot(corr~N,data=clines,type="p",pch=20,ylab="Correlation",
		xlab="N locations")
abline(h=mean(clines$corr),lwd=1,lty=3)

#both ordinary metanalysis and funell plot 
#indicate that net effects
#exists and correlation is positive
#what if we take phylogeny into account?


###code block C2

library(ape)
mammals <- read.nexus("mammals.nex")
plot(mammals,cex=0.75)

names(clines)[1]<-"animal"
prior.c2 <- list(R=list(V=1,nu=0.002),
			G=list(G1=list(V=1,nu=0.002)))
m.c13 <- MCMCglmm(FisherZ~1, verbose=F, prior=prior.c2,
			data=clines,
			mev=clines$mev,random=~animal,
			pedigree=mammals,
			nitt=150000,burnin=30000,thin=150)
plot(m.c13)
summary(m.c13)
diag(autocorr(m.c13$VCV)[2,,])
#should be ran for longer - >300000
#BUT effect we looked for disappeared...
#conclusions?


###code block C3

data(BTdata)
#we'll remove unkown sex
BTdata$sex[which(BTdata$sex=="UNK")]<-NA
BTdata$sex<-gdata::drop.levels(BTdata$sex)
#we remove UNK level from the variable

prior.c31 <-list(R=list(V=1,fix=1),
			G=list(G1=list(V=1,nu=0.002,
			alpha.mu=0,alpha.V=1000)))
prior.c32 <- list(R=list(V=1,fix=1),
			G=list(G1=list(V=1,nu=0.002)))
m.c4a <- MCMCglmm(sex~1,random=~dam,data=BTdata,
		family="categorical",prior=prior.c31,verbose=F,
		nitt=25000,burnin=5000,thin=25)
m.c4b <- MCMCglmm(sex~1,random=~dam,data=BTdata,
		family="categorical",prior=prior.c32,verbose=F,
		nitt=25000,burnin=5000,thin=25)
plot(mcmc.list(m.c4a$VCV[,"dam"],m.c4b$VCV[,"dam"]),
		col=c("red","green"))
effectiveSize(m.c4a$VCV[,"dam"])
effectiveSize(m.c4b$VCV[,"dam"])


#we might try equlize prior densities 
#for both models and use alternative priors
#proper for PE model and improper 
#flat for sd for the second model
prior.c31 <-list(R=list(V=1,fix=1),
			G=list(G1=list(V=1,nu=1,
			alpha.mu=0,alpha.V=1000)))
prior.c32 <- list(R=list(V=1,fix=1),
			G=list(G1=list(V=1e-16,nu=-1)))
m.c4a <- MCMCglmm(sex~1,random=~dam,data=BTdata,
		family="categorical",prior=prior.c31,verbose=F,
		nitt=25000,burnin=5000,thin=25)
m.c4b <- MCMCglmm(sex~1,random=~dam,data=BTdata,
		family="categorical",prior=prior.c32,verbose=F,
		nitt=25000,burnin=5000,thin=25)
plot(mcmc.list(m.c4a$VCV[,"dam"],m.c4b$VCV[,"dam"]),
	col=c("red","green"))
effectiveSize(m.c4a$VCV[,"dam"])
effectiveSize(m.c4b$VCV[,"dam"])



###code block C4

install.packages("rbugs");library(rbugs)
data(schools)

prior.c41 <- list(R=list(V=diag(schools$sd^2),fix=1),
			G=list(G1=list(V=1e-16,nu=-1)))
m.c41 <- MCMCglmm(estimate~1,random=~school,rcov=~idh(school):units,
			data=schools,prior=prior.c41,verbose=F)
sd1<-sqrt(m.c41$VCV[,1])
####OPTIONAL CODE####
hist(sd1[which(sd1<35)],breaks=40)
abline(h=50)
####END OF OPTIONAL CODE####

prior.c42 <- list(R=list(V=diag(schools$sd^2),fix=1),
			G=list(G1=list(V=1,nu=0.002)))
m.c42 <- MCMCglmm(estimate~1,random=~school,rcov=~idh(school):units,
			data=schools,prior=prior.c42,verbose=F)
sd2<-sqrt(m.c42$VCV[,1])
####OPTIONAL CODE####
hist(sd2[which(sd2<35)],breaks=40)
xv<-seq(1e-16,35,length=200)
dv<-MCMCpack::dinvgamma(xv,shape=0.001,scale=0.001)
lines(1000*sqrt(dv)~xv)
####END OF OPTIONAL CODE####

prior.c43 <- list(R=list(V=diag(schools$sd^2),fix=1),
			G=list(G1=list(V=1,nu=1,alpha.mu=0,alpha.V=25^2)))
m.c43 <- MCMCglmm(estimate~1,random=~school,rcov=~idh(school):units,
			data=schools,prior=prior.c43,verbose=F)
sd3<-sqrt(m.c43$VCV[,1])
####OPTIONAL CODE####
hist(sd3[which(sd2<35)],breaks=40)

dv<-(1/pi)*(25/((xv)^2+25^2))
lines(4000*dv~xv)
####END OF OPTIONAL CODE####


###code block C5

prior.c5a <- list(R=list(V=10,fix=1),G=list(G1=list(V=1,nu=1,
			alpha.mu=0, alpha.v=1000)))
m.c5a <- MCMCglmm(sex~1,random=~dam,data=BTdata,family="categorical",
		prior=prior.c5a,verbose=F,nitt=25000,burnin=5000,thin=25)

c2<-((16*sqrt(3))/(15*pi))^2 #rescaling to Vres=0
plot(mcmc.list(m.c4a$VCV[,1],m.c5a$VCV[,1]))
plot(mcmc.list(m.c4a$VCV[,1]/(1+c2*m.c4a$VCV[,"units"]),
	m.c5a$VCV[,1]/(1+c2*m.c5a$VCV[,"units"])))


effectiveSize(m.c4a$VCV[,1])
effectiveSize(m.c5a$VCV[,1])
#we might even increase effective size using different sampling method
m.c6a <- MCMCglmm(sex~1,random=~dam,data=BTdata,family="categorical",
		prior=prior.c5a,verbose=F,nitt=25000,burnin=5000,thin=25,slice=T)
effectiveSize(m.c6a$VCV[,1])



###code block C6

install.packages("pscl"); library(pscl)
data(bioChemists)
head(bioChemists)
#it seems there are lots of zeros in art
sum(bioChemists$art==0)/length(bioChemists$art)#more than 30% are zeros
#end we'd expect only 18% under Poisson process
ppois(0,mean(bioChemists$art))

prior.c71 <- list(R=list(V=diag(2),nu=0.002,fix=2))
m.c71 <- MCMCglmm(art~trait-1+at.level(trait,1):fem+at.level(trait,1):mar+
		at.level(trait,1):kid5+at.level(trait,1):phd+at.level(trait,1):ment,
		rcov=~idh(trait):units,data=bioChemists,prior=prior.c71,
		family="zipoisson",verbose=F)
plot(m.c71$Sol)
quantile(boot::inv.logit(m.c71$Sol[,2]/sqrt(1+c2)))
#or
quantile(plogis(m.c71$Sol[,2]/sqrt(1+c2)))

prior.c72<-list(R=list(V=1,nu=0.002))
m.c72 <- MCMCglmm(art~fem+mar+kid5+phd+ment,data=bioChemists,
		prior=prior.c72,family="poisson",verbose=F,saveX=T)
#posterior check of the need of ZIP model
ob.zer <- sum(bioChemists$art==0)
nr.zer <- 1:1000
for(i in 1:1000) {
	pred1 <- rnorm(915,(m.c72$X%*%m.c72$Sol[i,])@x,sqrt(m.c72$VCV[i]))
	nr.zer[i]<-sum(rpois(915,exp(pred1))==0)
}
hist(nr.zer,breaks=20)
abline(v=ob.zer,lwd=2)


###code block C7

m.c73 <- MCMCglmm(art~trait-1+at.level(trait,1):fem+at.level(trait,1):mar+
		at.level(trait,1):kid5+at.level(trait,1):phd+at.level(trait,1):ment,
		rcov=~idh(trait):units,data=bioChemists,prior=prior.c71,
		family="hupoisson",verbose=F)
plot(m.c73$Sol)
c2 <- (16*sqrt(3)/(15*pi))^2
#constant proportions of zeros across fixed effects
#since only intercept fitted in hurdle process
HPDinterval(boot::inv.logit(m.c73$Sol[,2]/sqrt(1+c2)))
#proportion of zeros in non-zero truncated Poisson distribution
HPDinterval(ppois(0,exp(m.c73$Sol[,1]+0.5*m.c73$VCV[,1])))
#CIs overlap


###code block C8

HPDinterval(ppois(0,exp(m.c73$Sol[,1]+m.c73$Sol[,3]+
0.5*m.c73$VCV[,1])))


###code block C9

m.c74 <- MCMCglmm(art~trait-1+at.level(trait,1:2):fem+at.level(trait,1):mar+
		at.level(trait,1):kid5+at.level(trait,1):phd+at.level(trait,1):ment,
		rcov=~idh(trait):units,data=bioChemists,prior=prior.c71,
		family="hupoisson",verbose=F)
HPDinterval(boot::inv.logit((m.c74$Sol[,2]+m.c74$Sol[,4])/sqrt(1+c2)))
#expected zeros from Hurdle proces shrunk
HPDinterval(ppois(0,exp(m.c74$Sol[,1]+m.c74$Sol[,3]+0.5*m.c74$VCV[,1])))
#but in males still there seem to be zero-deflation


###code block C10

m.c8 <- MCMCglmm(art~trait*(fem+mar+kid5+phd+ment),
		rcov=~trait:units, data=bioChemists,
		family="zapoisson", verbose=F)
summary(m.c8)


###code block C11

install.packages("survival");library(survival)
data(lung)
head(lung)
?lung


###code block C12

surlun <- with(na.omit(lung), Surv(time,status))
m.c91 <- survfit(surlun~1,data=na.omit(lung))
plot(m.c91,yscale=100) #general distribution of survival

m.c92 <- survfit(surlun~sex,data=na.omit(lung))
plot(m.c92,yscale=100,col=c("red","blue")) #sex-specific survival distribution
#and test therefor
survdiff(surlun~sex,data=na.omit(lung))


###code block C13


m.c93 <- coxph(surlun~age+ph.ecog+ph.karno+pat.karno,data=na.omit(lung),x=T)
m.c93
cox.zph(m.c93)#testing for assumptions of survival analysis



###code block C14

data(rats)
summary(rats)
ratss <- with(rats,Surv(time,status))
m.c94 <- survfit(ratss~rx,data=rats)
plot(m.c94,yscale=100,col=c("green","red"))
